# -*- coding: utf-8 -*-
# vi:ts=4:et
#
# $Date: 2005/04/16 07:14:34 $
# $Revision: 1.8 $
# =====================================================

"""pretty print library for math"""

import itertools
import pprint
import sys
import util

from cStringIO import StringIO

__all__     = [
               'pprint_whole_tableaux',
               'pprint_tableaux',
               'pprint_factorial',
               'pprint_partition',
               'pprint_skew_tableaux',
               ]


def pprint_whole_tableaux(part, tableaux):
    '''pretty print all the tableaux.
    '''
    buff = StringIO()
    buff.write('\n')
    buff.write(pprint_partition(part) + '\n')
    buff.write('='*40 + '\n')
    for table in tableaux:
        buff.write(pprint_tableaux(table))
        buff.write('-'*20 + '\n')
    return buff.getvalue()

def pprint_tableaux(tableaux):
    '''pretty print a tableau.

    >>> tableau = [
    ... [1, 2, 3],
    ... [4, 5, 6],
    ... [7],]
    >>> print pprint_tableaux(tableau)
     1  2  3
     4  5  6
     7
    <BLANKLINE>

    >>>
    '''
    buff = StringIO()
    for cor in tableaux:
        # convert int types into str types.
        # [1,2,3] -> ['1', '2', '3']
        cor = itertools.imap(lambda x : '%2s'%x, cor)
        buff.write(' '.join(cor) + '\n')

    # buff.write('\n'.join(' '.join(itertools.imap(lambda x:'%2s'%x, cor)) for cor in tableaux))

    return buff.getvalue()

def pprint_skew_tableaux(tableaux):
    '''pretty print a skew tableau.

    >>> tableau = [
    ... [None, None, 1],
    ... [None, 2, 3],
    ... [5],]
    >>> print pprint_skew_tableaux(tableau)
     *  *  1
     *  2  3
     5
    <BLANKLINE>
    >>>
    '''

    return pprint_tableaux(tableaux).replace('None', ' *')

def pprint_partition(part, prefix = '(', suffix = ')'):
    '''pretty print a partition.

    >>> part = (3, 2, 1, 1)
    >>> print pprint_partition(part)
    (3,2,1,1)
    >>>
    '''
    return prefix + util.int2str(part, ',') + suffix

def pprint_cycle(grp, ignore_single=True):
    '''pretty print a cycle.
    if ignore_single is True, don't pretty print a single element.
    >>> c = ((1,4), (2, ), (3,))
    >>> print pprint_cycle(c)
    (1 4)
    >>> print pprint_cycle(c, False)
    (1 4)(2)(3)
    '''

    if ignore_single:
        return '(' + ')('.join(util.int2str(c) for c in grp if len(c) != 1) + ')'
    else:
        return '(' + ')('.join(util.int2str(c) for c in grp) + ')'

def pprint_factorial(number):
    '''pretty print a factorial.

    >>> print pprint_factorial(10)
    1 x 2 x 3 x 4 x 5 x 6 x 7 x 8 x 9 x 10
    >>>
    '''
    seq = xrange(1, number+1)
    return util.int2str(seq, ' x ')

def _test():
    import doctest
    doctest.testmod()

if __name__ == '__main__':
    _test()
